@@ -384,7 +384,8 @@ static void
 FfmpegParseMetaData(const AVStream &stream,
 		    ReplayGainInfo &rg, MixRampInfo &mr)
 {
-	FfmpegParseMetaData(*stream.metadata, rg, mr);
+	if (stream.metadata != nullptr)
+		FfmpegParseMetaData(*stream.metadata, rg, mr);
 }
 
 static void
@@ -393,7 +394,9 @@ FfmpegParseMetaData(const AVFormatContext &format_context, int audio_stream,
 {
 	assert(audio_stream >= 0);
 
-	FfmpegParseMetaData(*format_context.metadata, rg, mr);
+	if (format_context.metadata != nullptr)
+		FfmpegParseMetaData(*format_context.metadata, rg, mr);
+
 	FfmpegParseMetaData(*format_context.streams[audio_stream],
 				    rg, mr);
 }
@@ -468,7 +471,7 @@ static bool
 IsSeekable(const AVFormatContext &format_context) noexcept
 {
 #if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(58, 6, 100)
-	return (format_context.ctx_flags & AVFMTCTX_UNSEEKABLE) != 0;
+	return (format_context.ctx_flags & AVFMTCTX_UNSEEKABLE) == 0;
 #else
 	(void)format_context;
 	return false;
@@ -520,9 +523,15 @@ FfmpegDecode(DecoderClient &client, InputStream *input,
 		return;
 	}
 
+#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(57, 25, 100)
+	const unsigned channels = codec_context->ch_layout.nb_channels;
+#else
+	const unsigned channels = codec_context->channels;
+#endif
+
 	const auto audio_format = CheckAudioFormat(codec_context->sample_rate,
 						   sample_format,
-						   codec_context->channels);
+						   channels);
 
 	const SignedSongTime total_time =
 		av_stream.duration != (int64_t)AV_NOPTS_VALUE
@@ -530,9 +539,8 @@ FfmpegDecode(DecoderClient &client, InputStream *input,
 		: FromFfmpegTimeChecked(format_context.duration, AV_TIME_BASE_Q);
 
 	client.Ready(audio_format,
-		     input
-		     ? input->IsSeekable()
-		     : IsSeekable(format_context),
+		     (input ? input->IsSeekable() : false)
+		     || IsSeekable(format_context),
 		     total_time);
 
 	FfmpegParseMetaData(client, format_context, audio_stream);
@@ -633,10 +641,17 @@ FfmpegScanStream(AVFormatContext &format_context, TagHandler &handler)
 						  AV_TIME_BASE_Q));
 
 	const auto &codec_params = *stream.codecpar;
+
+#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(57, 25, 100)
+	const unsigned channels = codec_params.ch_layout.nb_channels;
+#else
+	const unsigned channels = codec_params.channels;
+#endif
+
 	try {
 		handler.OnAudioFormat(CheckAudioFormat(codec_params.sample_rate,
 						       ffmpeg_sample_format(AVSampleFormat(codec_params.format)),
-						       codec_params.channels));
+						       channels));
 	} catch (...) {
 	}
 
